@@ -36,9 +36,6 @@
 #include "hw/pci/pci_ids.h"
 #include "hw/pci/pci_regs.h"
 
-/* TODO actually test the results and get rid of this */
-#define qmp_discard_response(s, ...) qobject_unref(qtest_qmp(s, __VA_ARGS__))
-
 /* Test images sizes in MB */
 #define TEST_IMAGE_SIZE_MB_LARGE (200 * 1024)
 #define TEST_IMAGE_SIZE_MB_SMALL 64
@@ -154,6 +151,7 @@ static void ahci_migrate(AHCIQState *from, AHCIQState *to, const char *uri)
 /**
  * Start a Q35 machine and bookmark a handle to the AHCI device.
  */
+G_GNUC_PRINTF(1, 0)
 static AHCIQState *ahci_vboot(const char *cli, va_list ap)
 {
     AHCIQState *s;
@@ -171,6 +169,7 @@ static AHCIQState *ahci_vboot(const char *cli, va_list ap)
 /**
  * Start a Q35 machine and bookmark a handle to the AHCI device.
  */
+G_GNUC_PRINTF(1, 2)
 static AHCIQState *ahci_boot(const char *cli, ...)
 {
     AHCIQState *s;
@@ -209,6 +208,7 @@ static void ahci_shutdown(AHCIQState *ahci)
  * Boot and fully enable the HBA device.
  * @see ahci_boot, ahci_pci_enable and ahci_hba_enable.
  */
+G_GNUC_PRINTF(1, 2)
 static AHCIQState *ahci_boot_and_enable(const char *cli, ...)
 {
     AHCIQState *ahci;
@@ -1424,89 +1424,6 @@ static void test_reset(void)
     ahci_shutdown(ahci);
 }
 
-static void test_reset_pending_callback(void)
-{
-    AHCIQState *ahci;
-    AHCICommand *cmd;
-    uint8_t port;
-    uint64_t ptr1;
-    uint64_t ptr2;
-
-    int bufsize = 4 * 1024;
-    int speed = bufsize + (bufsize / 2);
-    int offset1 = 0;
-    int offset2 = bufsize / AHCI_SECTOR_SIZE;
-
-    g_autofree unsigned char *tx1 = g_malloc(bufsize);
-    g_autofree unsigned char *tx2 = g_malloc(bufsize);
-    g_autofree unsigned char *rx1 = g_malloc0(bufsize);
-    g_autofree unsigned char *rx2 = g_malloc0(bufsize);
-
-    /* Uses throttling to make test independent of specific environment. */
-    ahci = ahci_boot_and_enable("-drive if=none,id=drive0,file=%s,"
-                                "cache=writeback,format=%s,"
-                                "throttling.bps-write=%d "
-                                "-M q35 "
-                                "-device ide-hd,drive=drive0 ",
-                                tmp_path, imgfmt, speed);
-
-    port = ahci_port_select(ahci);
-    ahci_port_clear(ahci, port);
-
-    ptr1 = ahci_alloc(ahci, bufsize);
-    ptr2 = ahci_alloc(ahci, bufsize);
-
-    g_assert(ptr1 && ptr2);
-
-    /* Need two different patterns. */
-    do {
-        generate_pattern(tx1, bufsize, AHCI_SECTOR_SIZE);
-        generate_pattern(tx2, bufsize, AHCI_SECTOR_SIZE);
-    } while (memcmp(tx1, tx2, bufsize) == 0);
-
-    qtest_bufwrite(ahci->parent->qts, ptr1, tx1, bufsize);
-    qtest_bufwrite(ahci->parent->qts, ptr2, tx2, bufsize);
-
-    /* Write to beginning of disk to check it wasn't overwritten later. */
-    ahci_guest_io(ahci, port, CMD_WRITE_DMA_EXT, ptr1, bufsize, offset1);
-
-    /* Issue asynchronously to get a pending callback during reset. */
-    cmd = ahci_command_create(CMD_WRITE_DMA_EXT);
-    ahci_command_adjust(cmd, offset2, ptr2, bufsize, 0);
-    ahci_command_commit(ahci, cmd, port);
-    ahci_command_issue_async(ahci, cmd);
-
-    ahci_set(ahci, AHCI_GHC, AHCI_GHC_HR);
-
-    ahci_command_free(cmd);
-
-    /* Wait for throttled write to finish. */
-    sleep(1);
-
-    /* Start again. */
-    ahci_clean_mem(ahci);
-    ahci_pci_enable(ahci);
-    ahci_hba_enable(ahci);
-    port = ahci_port_select(ahci);
-    ahci_port_clear(ahci, port);
-
-    /* Read and verify. */
-    ahci_guest_io(ahci, port, CMD_READ_DMA_EXT, ptr1, bufsize, offset1);
-    qtest_bufread(ahci->parent->qts, ptr1, rx1, bufsize);
-    g_assert_cmphex(memcmp(tx1, rx1, bufsize), ==, 0);
-
-    ahci_guest_io(ahci, port, CMD_READ_DMA_EXT, ptr2, bufsize, offset2);
-    qtest_bufread(ahci->parent->qts, ptr2, rx2, bufsize);
-    g_assert_cmphex(memcmp(tx2, rx2, bufsize), ==, 0);
-
-    ahci_free(ahci, ptr1);
-    ahci_free(ahci, ptr2);
-
-    ahci_clean_mem(ahci);
-
-    ahci_shutdown(ahci);
-}
-
 static void test_ncq_simple(void)
 {
     AHCIQState *ahci;
@@ -1675,9 +1592,9 @@ static void test_atapi_tray(void)
     rsp = qtest_qmp_receive(ahci->parent->qts);
     qobject_unref(rsp);
 
-    qmp_discard_response(ahci->parent->qts,
-                         "{'execute': 'blockdev-remove-medium', "
-                         "'arguments': {'id': 'cd0'}}");
+    qtest_qmp_assert_success(ahci->parent->qts,
+                             "{'execute': 'blockdev-remove-medium', "
+                             "'arguments': {'id': 'cd0'}}");
 
     /* Test the tray without a medium */
     ahci_atapi_load(ahci, port);
@@ -1687,16 +1604,18 @@ static void test_atapi_tray(void)
     atapi_wait_tray(ahci, true);
 
     /* Re-insert media */
-    qmp_discard_response(ahci->parent->qts,
-                         "{'execute': 'blockdev-add', "
-                         "'arguments': {'node-name': 'node0', "
-                                        "'driver': 'raw', "
-                                        "'file': { 'driver': 'file', "
-                                                  "'filename': %s }}}", iso);
-    qmp_discard_response(ahci->parent->qts,
-                         "{'execute': 'blockdev-insert-medium',"
-                         "'arguments': { 'id': 'cd0', "
-                                         "'node-name': 'node0' }}");
+    qtest_qmp_assert_success(
+        ahci->parent->qts,
+        "{'execute': 'blockdev-add', "
+        "'arguments': {'node-name': 'node0', "
+                      "'driver': 'raw', "
+                      "'file': { 'driver': 'file', "
+                                "'filename': %s }}}", iso);
+    qtest_qmp_assert_success(
+        ahci->parent->qts,
+        "{'execute': 'blockdev-insert-medium',"
+        "'arguments': { 'id': 'cd0', "
+                       "'node-name': 'node0' }}");
 
     /* Again, the event shows up first */
     qtest_qmp_send(ahci->parent->qts, "{'execute': 'blockdev-close-tray', "
@@ -2026,8 +1945,7 @@ int main(int argc, char **argv)
     qtest_add_func("/ahci/migrate/dma/halted", test_migrate_halted_dma);
 
     qtest_add_func("/ahci/max", test_max);
-    qtest_add_func("/ahci/reset/simple", test_reset);
-    qtest_add_func("/ahci/reset/pending_callback", test_reset_pending_callback);
+    qtest_add_func("/ahci/reset", test_reset);
 
     qtest_add_func("/ahci/io/ncq/simple", test_ncq_simple);
     qtest_add_func("/ahci/migrate/ncq/simple", test_migrate_ncq);
